from zope.component import testing

import doctest
import unittest


class TestWrapperUpdate(unittest.TestCase):
    def test_wrapper_update(self):
        from plone.indexer import indexer
        from zope.interface import Interface

        @indexer(Interface)
        def my_func(obj):
            """My custom docstring."""

        self.assertEqual(my_func.__doc__, "My custom docstring.")
        self.assertEqual(my_func.__module__, "plone.indexer.tests")
        self.assertEqual(my_func.__name__, "my_func")


def test_suite():
    return unittest.TestSuite(
        [
            doctest.DocFileSuite(
                "README.rst",
                package="plone.indexer",
                setUp=testing.setUp,
                tearDown=testing.tearDown,
            ),
            unittest.defaultTestLoader.loadTestsFromTestCase(TestWrapperUpdate),
        ]
    )


if __name__ == "__main__":
    unittest.main(defaultTest="test_suite")



from Acquisition import aq_base
from plone.indexer.interfaces import IIndexableObject
from plone.indexer.interfaces import IIndexableObjectWrapper
from plone.indexer.interfaces import IIndexer
from Products.CMFCore.utils import getToolByName
from Products.ZCatalog.interfaces import IZCatalog
from zope.component import adapter
from zope.component import queryMultiAdapter
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import providedBy
from zope.interface.declarations import getObjectSpecification
from zope.interface.declarations import ObjectSpecification
from zope.interface.declarations import ObjectSpecificationDescriptor


class WrapperSpecification(ObjectSpecificationDescriptor):
    """A __providedBy__ decorator that returns the interfaces provided by
    the wrapped object when asked.
    """

    def __get__(self, inst, cls=None):
        if inst is None:
            return getObjectSpecification(cls)
        else:
            provided = providedBy(inst._IndexableObjectWrapper__object)
            cls = type(inst)
            return ObjectSpecification(provided, cls)


@implementer(IIndexableObject, IIndexableObjectWrapper)
@adapter(Interface, IZCatalog)
class IndexableObjectWrapper:
    """A simple wrapper for indexable objects that will delegate to IIndexer
    adapters as appropriate.
    """

    __providedBy__ = WrapperSpecification()

    def __init__(self, object, catalog):
        self.__object = object
        self.__catalog = catalog
        self.__vars = {}

        portal_workflow = getToolByName(catalog, "portal_workflow", None)
        if portal_workflow is not None:
            self.__vars = portal_workflow.getCatalogVariablesFor(object)

    def _getWrappedObject(self):
        return self.__object

    def __str__(self):
        try:
            return self.__object.__str__()
        except AttributeError:
            return object.__str__(self)

    def __getattr__(self, name):
        # First, try to look up an indexer adapter
        indexer = queryMultiAdapter(
            (self.__object, self.__catalog),
            IIndexer,
            name=name,
        )
        if indexer is not None:
            return indexer()

        # Then, try workflow variables
        if name in self.__vars:
            return self.__vars[name]

        # first lets see if there is an attribute at all,
        # here we may already raise an AttributeError, which is fine
        value_or_callable = getattr(self.__object, name)
        try:
            # then lets see if the object provides the attribute directly,
            # w/o acquisition.
            getattr(aq_base(self.__object), name)
        except AttributeError:
            # it does not!
            # PythonScripts are the only way to add indexers TTW.
            # If there is a PythonScript acquired, that's fine:
            meta_type = getattr(value_or_callable, "meta_type", None)
            if meta_type == "Script (Python)":
                return value_or_callable
            raise
        # here we know it is a direct attribute.
        # we return the attribute acquistion wrapped in order to enable
        # callables to use acquisition.
        return value_or_callable



from functools import update_wrapper
from plone.indexer.interfaces import IIndexer
from zope.interface import implementer
from zope.interface.declarations import implementedBy
from zope.interface.declarations import Implements


@implementer(IIndexer)
class DelegatingIndexer:
    """An indexer that delegates to a given callable"""

    def __init__(self, context, catalog, callable):
        self.context = context
        self.catalog = catalog
        self.callable = callable

    def __call__(self):
        return self.callable(self.context)


class DelegatingIndexerFactory:
    """An adapter factory for an IIndexer that works by calling a
    DelegatingIndexer.
    """

    def __init__(self, callable):
        self.callable = callable
        self.__implemented__ = Implements(implementedBy(DelegatingIndexer))
        update_wrapper(self, callable)

    def __call__(self, object, catalog=None):
        return DelegatingIndexer(object, catalog, self.callable)



from setuptools import find_packages
from setuptools import setup

import os


version = "2.0.0"
description = "Hooks to facilitate managing custom index values in Zope 2/CMF applications"  # noqa
long_description = "\n\n".join(
    [
        open("README.rst").read(),
        open("CHANGES.rst").read(),
        open(os.path.join("plone", "indexer", "README.rst")).read(),
    ]
)


setup(
    name="plone.indexer",
    version=version,
    description=description,
    long_description=long_description,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 6.0",
        "Framework :: Plone :: Core",
        "Framework :: Zope :: 5",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    keywords="plone cmf zope catalog index",
    author="Plone Foundation",
    author_email="plone-developers@lists.sourceforge.net",
    url="https://pypi.org/project/plone.indexer",
    license="BSD",
    packages=find_packages(),
    namespace_packages=["plone"],
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.8",
    install_requires=[
        "setuptools",
        "Products.CMFCore",
    ],
)
